from datetime import datetime
from enum import Enum
from toloka.client.primitives.base import BaseTolokaObject
from typing import (
    Any,
    Dict,
    Optional
)

class DurationUnit(Enum):
    """An enumeration.
    """

    MINUTES = 'MINUTES'
    HOURS = 'HOURS'
    DAYS = 'DAYS'
    PERMANENT = 'PERMANENT'


class UserRestriction(BaseTolokaObject):
    """Allows you to control the performer's access to your projects and pools

    You can close user access to one or more projects. This allows you to control which users will perform tasks.
    For example, you can select users with a skill value below N and block them from accessing tasks.
    You can also unlock access.

    Attributes:
        user_id: Which performer is denied access.
        private_comment: A comment for you why access to this performer was restricted.
        will_expire: When access is restored. If you do not set the parameter, then the access restriction is permanent.
        id: The identifier of a specific fact of access restriction. Read only.
        created: Date and time when the fact of access restriction was created. Read only.

    Example:
        How you can lock access for one user on one project.

        >>> new_restrict = toloka_client.set_user_restriction(
        >>>     ProjectUserRestriction(
        >>>         user_id='1',
        >>>         private_comment='I dont like you',
        >>>         project_id='5'
        >>>     )
        >>> )
        ...

        And how you can unlock it.

        >>> toloka_client.delete_user_restriction(new_restrict.id)
        ...
    """

    class Scope(Enum):
        """Restriction scope

        * ALL_PROJECTS - All the requester's projects.
        * PROJECT - A single project (specify the project_id).
        * POOL - A pool (specify the pool_id).
        """

        SYSTEM = 'SYSTEM'
        ALL_PROJECTS = 'ALL_PROJECTS'
        PROJECT = 'PROJECT'
        POOL = 'POOL'

    def __init__(self, *, user_id: Optional[str] = None, private_comment: Optional[str] = None, will_expire: Optional[datetime] = None, id: Optional[str] = None, created: Optional[datetime] = None) -> None:
        """Method generated by attrs for class UserRestriction.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    user_id: Optional[str]
    private_comment: Optional[str]
    will_expire: Optional[datetime]
    id: Optional[str]
    created: Optional[datetime]


class AllProjectsUserRestriction(UserRestriction):
    """Forbid the performer from doing tasks from all your projects
    """

    def __init__(self, *, user_id: Optional[str] = None, private_comment: Optional[str] = None, will_expire: Optional[datetime] = None, id: Optional[str] = None, created: Optional[datetime] = None) -> None:
        """Method generated by attrs for class AllProjectsUserRestriction.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    user_id: Optional[str]
    private_comment: Optional[str]
    will_expire: Optional[datetime]
    id: Optional[str]
    created: Optional[datetime]


class PoolUserRestriction(UserRestriction):
    """Forbid the performer from doing tasks from a specific pool

    Attributes:
        pool_id: Pool identifier to which access will be denied.
    """

    def __init__(self, *, user_id: Optional[str] = None, private_comment: Optional[str] = None, will_expire: Optional[datetime] = None, id: Optional[str] = None, created: Optional[datetime] = None, pool_id: Optional[str] = None) -> None:
        """Method generated by attrs for class PoolUserRestriction.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    user_id: Optional[str]
    private_comment: Optional[str]
    will_expire: Optional[datetime]
    id: Optional[str]
    created: Optional[datetime]
    pool_id: Optional[str]


class ProjectUserRestriction(UserRestriction):
    """Forbid the performer from doing tasks from a specific project

    Attributes:
        project_id: Project identifier to which access will be denied.
    """

    def __init__(self, *, user_id: Optional[str] = None, private_comment: Optional[str] = None, will_expire: Optional[datetime] = None, id: Optional[str] = None, created: Optional[datetime] = None, project_id: Optional[str] = None) -> None:
        """Method generated by attrs for class ProjectUserRestriction.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    user_id: Optional[str]
    private_comment: Optional[str]
    will_expire: Optional[datetime]
    id: Optional[str]
    created: Optional[datetime]
    project_id: Optional[str]


class SystemUserRestriction(UserRestriction):
    """DEPRECATED
    """

    def __init__(self, *, user_id: Optional[str] = None, private_comment: Optional[str] = None, will_expire: Optional[datetime] = None, id: Optional[str] = None, created: Optional[datetime] = None) -> None:
        """Method generated by attrs for class SystemUserRestriction.
        """
        ...

    _unexpected: Optional[Dict[str, Any]]
    user_id: Optional[str]
    private_comment: Optional[str]
    will_expire: Optional[datetime]
    id: Optional[str]
    created: Optional[datetime]
